{%- if targets %}

| Name  | [GitLab include](https://docs.gitlab.com/ee/ci/yaml/includes.html) | [pre-commit hook](https://pre-commit.com/) | Description |
| ----- | ------- | ---- | ----------- |
{%- for target in targets %}
| [{{ target.name }}](#{{ target.name }}) | {% if target.gitlab_include %}✓{% endif %} | {% if target.precommit_hook %}✓{% endif %} | {{ target.brief }} |
{%- endfor -%}
{%- endif -%}
