{%- for group in groups | sort(attribute="name") %}

### `{{ group.name }}`

{%- if group.brief %}

{{ group.brief }}{% endif -%}

{%- if group.description %}

{{ group.description }}{%- endif -%}{%- endfor -%}
